/***************************************************************************
                          knetworkconfigparser.cpp  -  description
                             -------------------
    begin                : Mon Jan 13 2003
    copyright            : (C) 2003 by Juan Luis Baptiste
    email                : jbaptiste@merlinux.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knetworkconfigparser.h"

KNetworkConfigParser::KNetworkConfigParser(){
  networkInfo = new KNetworkInfo();
  runDetectionScript(QString::null);
}
/** Runs the gst backend to get network values. You can pass to the script a specific platform to load using the platform parameter.  */
void KNetworkConfigParser::runDetectionScript(QString platform){
	KDetectDistroDlg* dialog = new KDetectDistroDlg(0, 0);
	dialog->show();
	procDetect = new QProcess(this);
  procDetect->addArgument( locate("data",BACKEND_PATH) );
  if (platform != QString::null)
  {
    procDetect->addArgument( "--platform" );
    procDetect->addArgument( platform );
  }
  procDetect->addArgument( "--get" );
	connect( this, SIGNAL(readyLoadingNetworkInfo()), dialog, SLOT(close()) );
	connect( this, SIGNAL(errorDetectingPlatform()), dialog, SLOT(close()) );
  connect( procDetect, SIGNAL(processExited()), this, SLOT(readNetworkInfo()) );
  connect( procDetect, SIGNAL(readyReadStdout()),this, SLOT(concatXMLOutputSlot()));
  connect( procDetect, SIGNAL(readyReadStderr()),this, SLOT(readXMLErrSlot()));


 	if ( !procDetect->start() )
  {
// error handling
    KMessageBox::error(0,
                        i18n("Couldn't execute network detection scripts. Something is wrong with your installation."),
                        i18n("Couldn't launch network backend script"));
    dialog->close();
    exit(5);
	}
}

/** runs gst to find out the state of network devices.It runs the command:
$knetworkconf_home/backends/networkconf [--platform platform] -d list_ifaces. Well,
that was the idea, but -d list_ifaces doesn't return the boot proto nor other info,
so we got to do ot with --get */
void KNetworkConfigParser::listIfaces(QString platform){
	procDetect = new QProcess(this);
  procDetect->addArgument( locate("data",BACKEND_PATH) );
  if (platform != QString::null)
  {
    procDetect->addArgument( "--platform" );
    procDetect->addArgument( platform );
  }
  procDetect->addArgument( "--get" );
//  procDetect->addArgument( "-d" );
//  procDetect->addArgument( "list_ifaces" );
  
  connect( procDetect, SIGNAL(processExited()), this, SLOT(readListIfacesSlot()) );
  connect( procDetect, SIGNAL(readyReadStdout()),this, SLOT(concatXMLOutputSlot()));
  connect( procDetect, SIGNAL(readyReadStderr()),this, SLOT(readXMLErrSlot()));

  xmlOuput = "";
  xmlErr = "";
 	if ( !procDetect->start() )
  {
// error handling
    KMessageBox::error(0,
                        i18n("Couldn't execute network detection scripts. Something is wrong with your installation."),
                        i18n("Couldn't launch network backend script"));
    exit(5);
	}

}

/** No descriptions */
void KNetworkConfigParser::readListIfacesSlot(){
  QPtrList<KNetworkInterface> tempDeviceList;

  deviceList.clear();
  //The gst backend puts a \n at the beginning of the xml output, so
  //we have to erase it first before we parse it.
  xmlOuput = xmlOuput.section('\n',1);

  if (xmlErr.contains("platform_unsup::"))
  {
    connect( this, SIGNAL(readyLoadingSupportedPlatforms()), this, SLOT(showSupportedPlatformsDialogSlot()) );
    loadSupportedPlatforms();
    emit errorDetectingPlatform();
  }
  else
  {
    QString err;
    int x,y;
    QDomDocument doc( "network");
    if ( !doc.setContent( xmlOuput.utf8(),false,&err,&x,&y  ) )
    {
      KMessageBox::error(0,
									i18n("Couldn't parse the XML file."),
                      i18n("Error while listing network interfaces"));
        printf("error: %s %d,%d",err.latin1(),x,y);
    }
    QDomElement root = doc.documentElement();
    QDomNode node = root.firstChild();
    QString s;
    while( !node.isNull() )
    {
      if ( node.isElement() && node.nodeName() == "interface" )
      {
        QDomElement interface = node.toElement();
        KNetworkInterface *tempDevice = new KNetworkInterface();
        tempDevice = getInterfaceInfo(interface);
        //Don't include the loopback device
        if (tempDevice->getDeviceName().lower() != "lo")
          deviceList.append(tempDevice);
      }
      node = node.nextSibling();
    }
    loadDhcpDevices();
    networkInfo->setDeviceList(deviceList);
//     sleep(10);
    //Tell to interested parties when the network info is ready.
//    emit readyLoadingNetworkInfo();
    }
}

KNetworkConfigParser::~KNetworkConfigParser(){
}
/** Returns a list of the configured network interfaces in the system. */
QPtrList<KNetworkInterface> KNetworkConfigParser::loadNetworkDevices(){
  bool addDevice;
  QDir d;
  d.setPath(PATH_TO_RH_IFCFG_ETHx);
  d.setFilter( QDir::Files | QDir::NoSymLinks );

  const QFileInfoList *list = d.entryInfoList();
  QFileInfoListIterator it( *list );
  QFileInfo *fi;
  KNetworkInterface *interface = NULL;
  _numDevices = 0;
  //clear the list, just in case that there are already configured devices.
  deviceList.clear();
  while ( (fi = it.current()) != 0 )
  {
    addDevice = false;
    QString fileName = QString(fi->fileName());
    if (fileName.startsWith("ifcfg-") && !(fileName.contains(".bak")))
    {
      QFile f( d.filePath(fileName) );
        _numDevices++;
      if ( !f.open(IO_ReadOnly) )
      {
        KMessageBox::error(0,i18n("Error opening file"),
                            i18n("There was an error opening the file "+fileName+" for reading.Check permissions."));
      }
      else
      {
        QTextStream t( &f );        // use a text stream
        QString s;
        interface = new KNetworkInterface();        
        while (!t.eof())
        {
          addDevice = true;
          s = t.readLine();       // line of text excluding '\n'
          QString variable = s.section('=',0,0);
          QString value = s.section('=',1,1);
          if (value.startsWith("\""))
            value = value.section('"',1,1);
          if (variable.contains("DEVICE",false))
          {
            if (value.isEmpty())
            {
              addDevice = false;
              break;
            }  
            else  
              interface->setDeviceName(value);
          }  
          else if (variable.contains("BOOTPROTO",false))
            interface->setBootProto(value);
          else if (variable.contains("BROADCAST",false))
            interface->setBroadcast(value);
          else if (variable.contains("IPADDR",false))
            interface->setIpAddress(value);
          else if (variable.contains("NETMASK",false))
            interface->setNetmask(value);
          else if (variable.contains("NETWORK",false))
            interface->setNetwork(value);
          else if (variable.contains("ONBOOT",false))
            interface->setOnBoot(value);
        }
        if (addDevice)
          deviceList.append(interface);
        f.close();
      }      
    }
    ++it;
  }
  loadDhcpDevices();
  return deviceList;
}


  /** Reads the info in /etc/sysconfig/network and map it to a KRoutingInfo object. */
KRoutingInfo *KNetworkConfigParser::loadNetworkInfo(){
  routingInfo = NULL;
  QFile f( PATH_TO_RH_NETWORK_INFO );
  if ( !f.open(IO_ReadOnly) )
  {
    return routingInfo;
  }
  else
  {
    routingInfo = new KRoutingInfo();
    QTextStream t( &f );        // use a text stream
    QString s;
    while (!t.eof())
    {
      s = t.readLine();       // line of text excluding '\n'
      QString variable = s.section('=',0,0);
      variable = variable.upper();
      QString value = s.section('=',1,1);
      if (value.startsWith("\""))
        value = value.section('"',1,1);
      if (variable.compare("NETWORKING") == 0)
        routingInfo->setNetworkingEnabled(value);
      else if (variable.compare("FORWARD_IPV4") == 0)
        routingInfo->setForwardIPv4Enabled(value);
      else if (variable.compare("HOSTNAME") == 0)
        routingInfo->setHostName(value);
      else if (variable.compare("DOMAINNAME") == 0)
        routingInfo->setDomainName(value);
      else if (variable.compare("GATEWAY") == 0)
        routingInfo->setGateway(value);
      else if (variable.compare("GATEWAYDEV") == 0)
        routingInfo->setGatewayDevice(value);
      variable = "";
      value = "";
    }  
  }
  f.close();
  loadRoutingInfo();
  return routingInfo;
}
/** Reads /etc/resolv.conf and fills up a KDNSInfo object.
  It also reads /etc/hosts by calling loadKnownHostsInfo() method. */
KDNSInfo *KNetworkConfigParser::loadDNSInfo(){
  dnsInfo = NULL;
  QFile f( PATH_TO_RESOLV_CONF );
  if ( !f.open(IO_ReadOnly) )
  {
    return dnsInfo;
  }
  else
  {
    dnsInfo = new KDNSInfo();
    QTextStream t( &f );        // use a text stream
    QString s;
    QStringList searchList;
    QStringList serverList;
    
    while (!t.eof())
    {
      s = t.readLine();       // line of text excluding '\n'
      s = s.simplifyWhiteSpace();
      QString variable = s.section(' ',0,0);
      variable = variable.upper();
      QString value = s.section(' ',1,1);
      if (variable.compare("NAMESERVER") == 0)
        serverList.append(value);
//      else if (variable.compare("DOMAIN") == 0)
//        dnsInfo->setDomainName(value);
      else if (variable.compare("SEARCH") == 0)
        searchList.append(value);
      variable = "";
      value = "";
    }
    dnsInfo->setNameServers(serverList);
    dnsInfo->setSearchDomains(searchList);
    knownHostsList = loadKnownHostsInfo();
    dnsInfo->setKnownHostsList(knownHostsList);
  }
  f.close();

  return dnsInfo;
}
/** return tyhe number of configured devices. */
unsigned KNetworkConfigParser::numDevices(){
  return _numDevices;
}
/** Creates new ifcfg-[INTERFACE] files in /etc/sysconfig/network-scrpts with the new info,
and saves backup (.bak) files of the actual configuration files.
Returns true on success, false on failure. */
bool KNetworkConfigParser::saveDevicesInfo(QPtrList<KNetworkInterface> deviceList){
  QPtrListIterator<KNetworkInterface> it( deviceList );
  KNetworkInterface *device;
  dhcpDevice = false;
  while ( (device = it.current()) != 0 )
  {
    ++it;
    QString fileName = "";
    fileName = fileName.sprintf("%sifcfg-%s",PATH_TO_RH_IFCFG_ETHx, device->getDeviceName().latin1());
    QFile file( fileName );
    if (file.exists())
      if (!backupFile(fileName))
        return false;
    if ( file.open( IO_WriteOnly ) )
    {
      QTextStream stream( &file );
      stream << "##### FILE GENERATED BY KNetworkConf " << programVersion << " #####\n\n";
      stream << "DEVICE=" << device->getDeviceName() << "\n";
      stream << "BOOTPROTO=" << device->getBootProto() << "\n";
      if ((device->getBootProto().compare("dhcp") != 0) || (device->getBootProto().compare("bootp") != 0))
      {
        stream << "IPADDR=" << device->getIpAddress() << "\n";
        stream << "NETMASK=" << device->getNetmask() << "\n";
        stream << "NETWORK=" << device->getNetwork() << "\n";
        stream << "BROADCAST=" << device->getBroadcast() << "\n";
      }
      else
        dhcpDevice = true;
      stream << "ONBOOT=" << device->getOnBoot() << "\n";
      
      file.close();
      fileName = "";
    }
    else
      return false;
  }
  return true;
}
/** Saves the info of the file /etc/sysconfig/network and backups the acutal file.
Returns true on success, false on failure. */
bool KNetworkConfigParser::saveNetworkInfo(KRoutingInfo *_routingInfo){
  QString fileName = "";
    fileName = fileName.sprintf("%s",PATH_TO_RH_NETWORK_INFO);
  QFile file( fileName );
  if (file.exists())
    if (!backupFile(fileName))
      return false;
    if ( file.open( IO_WriteOnly ) )
    {
      QTextStream stream( &file );
      stream << "##### FILE GENERATED BY KNetworkConf " << programVersion << " #####\n\n";
//      stream << "NETWORKING=" << _routingInfo->isNetworkingEnabled() << "\n";
      stream << "NETWORKING=YES\n";      
      stream << "FORWARD_IPV4=" << _routingInfo->isForwardIPv4Enabled() << "\n";
      stream << "HOSTNAME=" << _routingInfo->getHostName() << "\n";
      stream << "DOMAINNAME=" << _routingInfo->getDomainName() << "\n";
      if (!dhcpDevice)
      {
        stream << "GATEWAY=" << _routingInfo->getGateway() << "\n";
        stream << "GATEWAYDEV=" << _routingInfo->getGatewayDevice() << "\n";
      }
      file.close();
    }
    else
      return false;
  return true;
}
/** Saves the DNS info in /etc/resolv.conf. For now it only has support for
'nameserver' and 'domain' keywords. It saves a backup of the actual file first.
Returns true on success, false on failure. */
bool KNetworkConfigParser::saveDNSInfo(KDNSInfo *dnsInfo){
  QString fileName = "";
  fileName = fileName.sprintf(PATH_TO_RESOLV_CONF);
  QFile file( fileName );
  if (file.exists())
    if (!backupFile(fileName))
      return false;
    if ( file.open( IO_WriteOnly ) )
    {
      QTextStream stream( &file );
//      stream << "##### FILE GENERATED BY KNetworkConf " << programVersion.latin1() << " #####\n\n";
      stream << "domain " << dnsInfo->getDomainName() << "\n\n";
      QStringList list;
      list = dnsInfo->getNameServers();
      for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
        stream << "nameserver " << *it << "\n";
      file.close();
      fileName = "";
      saveKnownHostsInfo(dnsInfo->getKnownHostsList());
    }
    else
      return false;
  return true;

}
    /*Saves the /etc/hosts file.*/
bool KNetworkConfigParser::saveKnownHostsInfo(QPtrList<KKnownHostInfo> hostList){
  QString fileName = "";
  fileName = fileName.sprintf(PATH_TO_HOSTS);
  QFile file( fileName );
  if (file.exists())
    if (!backupFile(fileName))
      return false;
    if ( file.open( IO_WriteOnly ) )
    {
      QTextStream stream( &file );
      stream << "##### FILE GENERATED BY KNetworkConf " << programVersion.latin1() << " #####\n\n";
      QPtrListIterator<KKnownHostInfo> it(hostList);
      KKnownHostInfo *host;
      while ((host = it.current()) != 0)
      {
        ++it;
        stream << host->getIpAddress() << "\t" << host->getHostName() << "\t" << host->getAliases() << endl;
      }
      file.close();
      fileName = "";
    }
    else
      return false;
  return true;

}
/** No descriptions */
void KNetworkConfigParser::setProgramVersion(QString ver){
  KNetworkConfigParser::programVersion = ver;
}
/** Saves a backup of the specified file 'file' with the '.bak' extension. */
bool KNetworkConfigParser::backupFile(QString fileName){
  QFile file(fileName);
  QString newFileName = "";
  newFileName = fileName.sprintf("%s.bak",file.name().latin1());
  
//  newFileName = newFileName.sprintf("%s.bak", file.name().latin1());
  QFile newFile( newFileName );
  if ( file.open( IO_ReadOnly ) && newFile.open( IO_WriteOnly ))
  {
    QTextStream originalFile( &file );
    QTextStream backupFile( &newFile );
    
    QString s;
    while ( !originalFile.eof() )
    { // until end of file...
      s = originalFile.readLine();       // line of text excluding '\n'
      backupFile << s << "\n";
    }    
    file.close();
    newFile.close();
    newFileName = "";
  }
  else
    return false;
  return true;
}
/** Loads the info of the device in the case that the boot protocol is dhcp or bootp,
because in this case the information is not contained config files. */
void KNetworkConfigParser::loadDhcpDevices(){
  proc = new QProcess(this);
  proc->addArgument( "/sbin/ifconfig" );

  connect( proc, SIGNAL(readyReadStdout()), this, SLOT(readIfconfigOutput()) );
  connect( proc, SIGNAL(processExited()), this, SLOT(fillDhcpDevices()) );
                

  if ( !proc->start() )
  {
    // error handling
    KMessageBox::error(0,
                        i18n("Could not launch command ifconfig"),
                        i18n("There was an error executing external program"));

  }
}
void KNetworkConfigParser::loadDhcpDevice(KNetworkInterface *device){
  proc = new QProcess(this);
  proc->addArgument( "/sbin/ifconfig" );

  connect( proc, SIGNAL(readyReadStdout()), this, SLOT(readIfconfigOutput()) );
  connect( proc, SIGNAL(processExited()), this, SLOT(fillDhcpDevice(device)) );


  if ( !proc->start() )
  {
    // error handling
    KMessageBox::error(0,
                        i18n("Could not launch command ifconfig"),
                        i18n("There was an error executing external program"));

  }
}

void KNetworkConfigParser::readIfconfigOutput(){
  QString s = proc->readStdout();
  ifconfigOutput = s;
}

void KNetworkConfigParser::fillDhcpDevices(){
  KNetworkInterface *device;
  for (device = deviceList.first(); device; device = deviceList.next())
  {
    QString temp = ifconfigOutput.section(device->getDeviceName(),1,1);
    QString temp2 = temp.section("UP",0,0); //two firts lines of the device info.
    QString temp3 = temp2.section("\n",0,0); //Link encap:Ethernet  HWaddr 00:00:21:C5:99:A0
    QString temp4 = temp2.section("\n",1,1); //inet addr:192.168.1.1  Bcast:192.255.255.255  Mask:255.0.0.0
    temp3 = temp3.stripWhiteSpace();
    temp4 = temp4.stripWhiteSpace();    
    QString temp5 = temp3.section(" ",4,4); //00:00:21:C5:99:A0
    QString temp6 = temp4.section(" ",1,1); // addr:192.168.1.1
    temp6 = temp6.section(":",1,1); //192.168.1.1
    QString temp7 = temp4.section(" ",3,3); //Bcast:192.255.255.255
    temp7 = temp7.section(":",1,1); //192.255.255.255   
    QString temp8 = temp4.section(" ",5,5); // Mask:255.0.0.0
    temp8 = temp8.section(":",1,1); //255.0.0.0

    //If the ip address is empty it must be a dhcp interface, so fill these fields:
    if (device->getIpAddress().isEmpty())
    {
      device->setIpAddress(temp6);
      device->setBroadcast(temp7);
      device->setNetmask(temp8);
      device->setMacAddress(temp5);  
      if (!temp6.isEmpty() && !temp8.isEmpty())
        device->setNetwork(KAddressValidator::calculateNetwork(temp6,temp8));
    }    
/*    printf("temp3= %s<\n
            temp4= %s<\n                  
            temp5= %s<\n
            temp6= %s<\n
            temp7= %s<\n            
            temp8= %s<\n",temp3.latin1(),temp4.latin1(),temp5.latin1(),temp6.latin1(),temp7.latin1(),temp8.latin1());*/
  }
//  emit readyIfconfigOutputSignal();
    emit readyLoadingNetworkInfo();
}
void KNetworkConfigParser::fillDhcpDevice(KNetworkInterface *device){
    QString temp = ifconfigOutput.section(device->getDeviceName(),1,1);
    QString temp2 = temp.section("UP",0,0); //two firts lines of the device info.
    QString temp3 = temp2.section("\n",0,0); //Link encap:Ethernet  HWaddr 00:00:21:C5:99:A0
    QString temp4 = temp2.section("\n",1,1); //inet addr:192.168.1.1  Bcast:192.255.255.255  Mask:255.0.0.0
    temp3 = temp3.stripWhiteSpace();
    temp4 = temp4.stripWhiteSpace();
    QString temp5 = temp3.section(" ",4,4); //00:00:21:C5:99:A0
    QString temp6 = temp4.section(" ",1,1); // addr:192.168.1.1
    temp6 = temp6.section(":",1,1); //192.168.1.1
    QString temp7 = temp4.section(" ",3,3); //Bcast:192.255.255.255
    temp7 = temp7.section(":",1,1); //192.255.255.255
    QString temp8 = temp4.section(" ",5,5); // Mask:255.0.0.0
    temp8 = temp8.section(":",1,1); //255.0.0.0

    device->setIpAddress(temp6);
    device->setBroadcast(temp7);
    device->setNetmask(temp8);
    device->setMacAddress(temp5);
    if (!temp6.isEmpty() && !temp8.isEmpty())
      device->setNetwork(KAddressValidator::calculateNetwork(temp6,temp8));
/*    printf("temp3= %s<\n
            temp4= %s<\n
            temp5= %s<\n
            temp6= %s<\n
            temp7= %s<\n
            temp8= %s<\n",temp3.latin1(),temp4.latin1(),temp5.latin1(),temp6.latin1(),temp7.latin1(),temp8.latin1());*/
  emit readyIfconfigOutputSignal();
}  
/** Reads /proc/net/route looking for the default gateway. */
void KNetworkConfigParser::loadRoutingInfo(){
//  routingInfo = NULL;
  QFile f( "/proc/net/route");
  if ( !f.open(IO_ReadOnly) )
  {
//    return routingInfo;
    KMessageBox::error(0,
                        i18n("Could not open file"),
                        i18n("Couldn't open file /proc/net/route!"));
  }
  else
  {
//    routingInfo = new KRoutingInfo();
    QTextStream t( &f );        // use a text stream
    QString s;
    while (!t.eof())
    {
      s = t.readLine();       // line of text excluding '\n'
      QString interface = s.section('\t',0,0);      
      QString destination = s.section('\t',1,1);
      QString gw = s.section('\t',2,2);
        
      if (destination == "00000000")
      {
        routingInfo->setGateway(hexIPv4ToDecIPv4(gw));
//        routingInfo->setGateway(gw);
        routingInfo->setGatewayDevice(interface);
      }  
    }
  }
  f.close();
}
QString
KNetworkConfigParser::hexIPv4ToDecIPv4(QString hex)
{
  bool ok;
  QString dec = "";
  QString dec2 = "";
  QString temp = "";
  QString temp2 = "";
  temp = hex.mid(6,2);
  temp2 = temp2.setNum(temp.toInt(&ok,16));
  dec.append(temp2);
  dec.append('.');
  temp = hex.mid(4,2);
  temp2 = temp2.setNum(temp.toInt(&ok,16));  
  dec.append(temp2);
  dec.append('.');
  temp = hex.mid(2,2);
  temp2 = temp2.setNum(temp.toInt(&ok,16));  
  dec.append(temp2);
  dec.append('.');
  temp = hex.mid(0,2);
  temp2 = temp2.setNum(temp.toInt(&ok,16));
  dec.append(temp2);

  dec2 = dec;
  if (ok)
    return dec2;
  else
    return "";
}
#include "knetworkconfigparser.moc"
/** Loads the network info contained in netinfo.conf and fills a 
 QPrtList<KNetworkInterface> with the network devices found,
a KDNSInfo with the DNS servers, and a KRoutingInfo with info of
the default gw and routes info.
 */
KNetworkInfo * KNetworkConfigParser::loadNetworkInfo2(){
  networkInfo = NULL;
  dnsInfo = NULL;
  routingInfo = NULL;
  QPtrList<KNetworkInterface> tempDeviceList;
  
  
  QFile f( PATH_TO_CONF_FILE );
  if ( !f.open(IO_ReadOnly) )
  {
    return networkInfo;
  }
  else
  {
    networkInfo = new KNetworkInfo();
    routingInfo = new KRoutingInfo();
    dnsInfo = new KDNSInfo();
    dnsInfo = loadDNSInfo();
    QTextStream t( &f );        // use a text stream
    QString s;
    QString variable;
    QString value;

    while (!t.eof())
    {
      s = t.readLine();       // line of text excluding '\n'
      s = s.simplifyWhiteSpace();
      s = s.lower();
      variable = s.section('=',0,0);
      value = s.section('=',1,1);      
      if (s.section(' ',0,0).lower() == "iface")
      {
        KNetworkInterface *tempDevice = new KNetworkInterface();
        tempDevice->setDeviceName(s.section(' ',1,1));
        tempDevice->setBootProto(s.section(' ',2,2));        
        int n = 0;
        int max_params = 0;
        if (tempDevice->getBootProto().lower() == "static")
          max_params = 5;
        else if (tempDevice->getBootProto().lower() == "dhcp")
          max_params = 1;  
        while (n < max_params)
        {
          s = t.readLine();       // line of text excluding '\n'
          s = s.simplifyWhiteSpace();
          s = s.lower();
          variable = s.section('=',0,0);
          value = s.section('=',1,1);
          
          if (!s.startsWith("#"))
          {
            if (variable == "address")  
              tempDevice->setIpAddress(value);
            else if (variable == "netmask")
              tempDevice->setNetmask(value);
            else if (variable == "network")
              tempDevice->setNetwork(value);
            else if (variable == "broadcast")
              tempDevice->setBroadcast(value);
            else if (variable == "onboot")
              tempDevice->setOnBoot(value);

            n++;
            variable = "";
            value = "";
          }
        }
        n = 0;
        if (KAddressValidator::isValidIPAddress(tempDevice->getIpAddress()) &&
            KAddressValidator::isNetmaskValid(tempDevice->getNetmask()) &&
            tempDevice->getDeviceName() == "static")
        {
          if (tempDevice->getNetwork() == "")
            tempDevice->setNetwork(KAddressValidator::calculateNetwork(tempDevice->getIpAddress(),tempDevice->getNetmask()));
          if (tempDevice->getBroadcast() == "")            
            tempDevice->setBroadcast(KAddressValidator::calculateBroadcast(tempDevice->getIpAddress(),tempDevice->getNetmask()));

            //Should we validate that the network and broadcast are valid? and if not show a warning message?
        }
        
          tempDeviceList.append(tempDevice);
      }
      else if (variable == "gw_address")
      {
            routingInfo->setGateway(value);
      }
      else if (variable == "gw_device")
      {
        routingInfo->setGatewayDevice(value);
      }
      else if (variable == "forward_ipv4")
      {
        routingInfo->setForwardIPv4Enabled(value);
      }
      else if (variable == "domain_name")
      {
        dnsInfo->setDomainName(value);
      }
      else if (variable == "host_name")
      {
        dnsInfo->setMachineName(value);
      }
      else if (variable == "network_script")
      {
        networkInfo->setNetworkScript(value);
      }
      
    }

    loadDhcpDevices();
    
    networkInfo->setDeviceList(tempDeviceList);
    loadRoutingInfo();
    networkInfo->setRoutingInfo(routingInfo);
    networkInfo->setDNSInfo(dnsInfo);    
  }
  f.close();
  return networkInfo;
}
/** Saves the new network configuration. */
bool KNetworkConfigParser::saveNetworkInfo2(KNetworkInfo *networkInfo)
{
  QString fileName = "";
  fileName = fileName.sprintf("%s",PATH_TO_CONF_FILE);
  QPtrList<KNetworkInterface> list = networkInfo->getDeviceList();
  QPtrListIterator<KNetworkInterface> it(list);
  KNetworkInterface *device;

  //TEMPORARLY ADDED TO RELEASE VERSION 0.4.1, WHILE NEW DISTRO ABSTRACTION LAYER IS READY!!!
  routingInfo = networkInfo->getRoutingInfo();
  dnsInfo = networkInfo->getDNSInfo();
  saveNetworkInfo(routingInfo);
  saveDNSInfo(dnsInfo);
  saveDevicesInfo(networkInfo->getDeviceList());
  dhcpDevice = false;
  //END
//  fileName = fileName.sprintf(QDir::homeDirPath()+"/network");
  QFile file( fileName );
  if (file.exists())
    if (!backupFile(fileName))
      return false;
    if ( file.open( IO_WriteOnly ) )
    {
      QTextStream stream( &file );
      stream << "##### FILE GENERATED BY KNetworkConf " << programVersion.latin1() << " #####\n\n";
      while ( (device = it.current()) != 0 )
      {
        ++it;
        stream << "iface " << device->getDeviceName() << " " << device->getBootProto().lower() << "\n";
        if (device->getBootProto().lower().compare("static") == 0)
        {
          stream << "\taddress=" << device->getIpAddress() << "\n";
          stream << "\tnetmask=" << device->getNetmask() << "\n";
          stream << "\tnetwork=" << device->getNetwork() << "\n";
          stream << "\tbroadcast=" << device->getBroadcast() << "\n";
        }
        stream << "\tonboot=" << device->getOnBoot() << "\n\n";
      }
      KRoutingInfo *routingInfo = networkInfo->getRoutingInfo();
      stream << "gw_address=" << routingInfo->getGateway() << "\n";
      stream << "gw_device=" << routingInfo->getGatewayDevice() << "\n";
      stream << "forward_ipv4=" << routingInfo->isForwardIPv4Enabled() << "\n\n";


      KDNSInfo *dnsInfo = networkInfo->getDNSInfo();
      stream << "domain_name=" << dnsInfo->getDomainName() << "\n";
      stream << "host_name=" << dnsInfo->getMachineName() << "\n\n";
      stream << "network_script=" << networkInfo->getNetworkScript() << "\n";
      saveDNSInfo(dnsInfo);
      file.close();
    }
    else
      return false;
  return true;

}
bool KNetworkConfigParser::saveNetworkInfo3(KNetworkInfo *networkInfo)
{
  QPtrList<KNetworkInterface> devList = networkInfo->getDeviceList();
  QStringList nameServerList = dnsInfo->getNameServers();
  QPtrListIterator<KNetworkInterface> devIt(devList);

  KNetworkInterface *device;
  routingInfo = networkInfo->getRoutingInfo();
  dnsInfo = networkInfo->getDNSInfo();

  //Start xml file
  QDomDocument doc( "network []" );
  QDomProcessingInstruction instr = doc.createProcessingInstruction("xml","version=\"1.0\" ");
  doc.appendChild(instr);
  QDomElement root = doc.createElement( "network" );
  doc.appendChild( root );

  //Add routing info
  QDomElement tag = doc.createElement( "gateway" );
  root.appendChild( tag );
  QDomText t = doc.createTextNode( routingInfo->getGateway() );
  tag.appendChild( t );

  tag = doc.createElement( "gatewaydev" );
  root.appendChild( tag );
  t = doc.createTextNode( routingInfo->getGatewayDevice() );
  tag.appendChild( t );

  tag = doc.createElement( "hostname" );
  root.appendChild( tag );
  t = doc.createTextNode( dnsInfo->getMachineName() );
  tag.appendChild( t );
  tag = doc.createElement( "domain" );
  root.appendChild( tag );
  t = doc.createTextNode( dnsInfo->getDomainName() );
  tag.appendChild( t );

    for ( QStringList::Iterator it = nameServerList.begin(); it != nameServerList.end(); ++it ) {
      tag = doc.createElement( "nameserver" );
  		root.appendChild( tag );
  		t = doc.createTextNode( *it );
  		tag.appendChild( t );
    }
  //Add de network interfaces list
  while ( (device = devIt.current()) != 0 )
  {
  	++devIt;
		tag = doc.createElement( "interface" );
  	root.appendChild( tag );
		QDomElement innerTag;
    if ((device->getBootProto().lower() != "dhcp") && (device->getBootProto().lower() != "bootp"))
    {
			innerTag = doc.createElement( "address" );
  		tag.appendChild( innerTag );
			t = doc.createTextNode( device->getIpAddress() );
  		innerTag.appendChild( t );

      innerTag = doc.createElement( "broadcast" );
  		tag.appendChild( innerTag );
			t = doc.createTextNode( device->getBroadcast() );
  		innerTag.appendChild( t );

      innerTag = doc.createElement( "netmask" );
	  	tag.appendChild( innerTag );
			t = doc.createTextNode( device->getNetmask() );
  		innerTag.appendChild( t );

    	innerTag = doc.createElement( "network" );
  		tag.appendChild( innerTag );
			t = doc.createTextNode( device->getNetwork() );
  		innerTag.appendChild( t );
			innerTag = doc.createElement( "name" );
  		tag.appendChild( innerTag );
			t = doc.createTextNode( device->getDescription() );
  		innerTag.appendChild( t );
    }
    innerTag = doc.createElement( "auto" );
  	tag.appendChild( innerTag );
    if (device->getOnBoot().lower() == "yes")
  		t = doc.createTextNode( "1" );
    else
  		t = doc.createTextNode( "0" );
    
  	innerTag.appendChild( t );

    innerTag = doc.createElement( "bootproto" );
  	tag.appendChild( innerTag );
    if (device->getBootProto().lower() == "manual")
      t = doc.createTextNode( "none" );
    else
		  t = doc.createTextNode( device->getBootProto().lower() );
  	innerTag.appendChild( t );


		innerTag = doc.createElement( "dev" );
  	tag.appendChild( innerTag );
		t = doc.createTextNode( device->getDeviceName() );
  	innerTag.appendChild( t );

    innerTag = doc.createElement( "enabled" );
  	tag.appendChild( innerTag );
		t = doc.createTextNode( "1" );
  	innerTag.appendChild( t );

    innerTag = doc.createElement( "file" );
  	tag.appendChild( innerTag );
		t = doc.createTextNode( device->getDeviceName() );
  	innerTag.appendChild( t );
  }
  //Save /etc/resolv.conf info. Should we do it through gst?
  saveDNSInfo(networkInfo->getDNSInfo());

  QString xml = doc.toString();
  qDebug("--set XML:\n%s",xmlOuput.latin1());
  procSaveNetworkInfo = new QProcess(this);
  procSaveNetworkInfo->addArgument( locate("data",BACKEND_PATH) );
  if (!networkInfo->getPlatformName().isEmpty())
  {
    procSaveNetworkInfo->addArgument( "--platform" );
    procSaveNetworkInfo->addArgument( networkInfo->getPlatformName() );
  }  
  procSaveNetworkInfo->addArgument( "--set" );

//  KDetectDistroDlg* dialog = new KDetectDistroDlg(0, 0, true,QDialog::WStyle_Customize|QDialog::WStyle_NormalBorder|QDialog::WStyle_Title|QDialog::WStyle_SysMenu); //made it semi-modal
  KDetectDistroDlg* dialog = new KDetectDistroDlg(0, 0);
  dialog->text->setText(i18n("%1Please wait while reloading the Network...%2").arg("<center>").arg("</center>"));
	dialog->show();

  xmlOuput = "";
	connect( this, SIGNAL(readyLoadingNetworkInfo()), dialog, SLOT(close()) );
  connect(procSaveNetworkInfo,SIGNAL(readyReadStdout()),this,SLOT(readFromStdoutSaveNetworkInfo()));
  connect(procSaveNetworkInfo,SIGNAL(wroteToStdin()),this,SLOT(sendNetworkInfoSavedSignalSlot()));
  connect(procSaveNetworkInfo,SIGNAL(processExited()),this,SLOT(runDetectionScriptSlot()));
             
  if ( !procSaveNetworkInfo->start() )
  {
    KMessageBox::error(0,
                        i18n("Couldn't execute network saving scripts. Something is wrong with your installation."),
                        i18n("Couldn't launch network backend script"));
    return false;
  }

  procSaveNetworkInfo->writeToStdin(xml);
  
  return true;
}

/** Verifies the exit code of the detection shell script to know if it had success or not
    detecting the Linux distro is running on.*/
void KNetworkConfigParser::verifyDetectStatusSlot(){
  QFile file( "/tmp/network.xml" );
	QString s;

  s.append(procDetect->readStdout());
  if (file.exists())
  {
    file.remove();
  }
  if ( file.open(IO_WriteOnly | IO_Append) )
  {
    QTextStream stream( &file );
    stream << s;
    file.close();
  }
  else
    KMessageBox::detailedError(0,
											i18n("There was an error detecting your current distro."),
                        i18n(s),
                        i18n("Error"));
	exit(1);
}
/** Reads /etc/hosts file and returns a QPtrList<KKnownHostInfo>. */
QPtrList<KKnownHostInfo> KNetworkConfigParser::loadKnownHostsInfo(){
  KKnownHostInfo * knownHostInfo;
  QPtrList<KKnownHostInfo> list;
  QFile f( PATH_TO_HOSTS );
  if ( !f.open(IO_ReadOnly) )
  {
    return list;
  }
  else
  {
    QTextStream t( &f );        // use a text stream
    QString s;
    QString ipAddr;
    QString aliases;
    QString hostName;
    
    while (!t.eof())
    {
      knownHostInfo = new KKnownHostInfo();
      s = t.readLine();       // line of text excluding '\n'
      s = s.simplifyWhiteSpace();
      if (!s.startsWith("#"))
      {
        ipAddr = s.section(' ',0,0);
        hostName = s.section(' ',1,1);
        aliases = s.section(' ',2);
        knownHostInfo->setIpAddress(ipAddr);
        knownHostInfo->setHostName(hostName);
        knownHostInfo->setAliases(aliases);

        list.append(knownHostInfo);
      }  
    }  
  }  
  f.close();
  
  return list;
}
/** Loads the network info from a xml file generated by the gnome system tools
network backends that are included with this app. */
KNetworkInfo * KNetworkConfigParser::loadNetworkInfo3(){
     
    return networkInfo;
}
/** Parses all of the <interface>...</interface> entries in the xml configuration file. */
KNetworkInterface * KNetworkConfigParser::getInterfaceInfo(QDomElement interface){
  QDomNode node = interface.firstChild();
  KNetworkInterface *tempDevice = new KNetworkInterface();
  tempDevice->setDescription(i18n("Ethernet Network Device"));
  
  while ( !node.isNull() )
  {
    if ( node.isElement() )
    {
      if ( node.nodeName() == "auto" )
      {
        QDomElement e = node.toElement();
        if (e.text() == "1")
          tempDevice->setOnBoot("yes");
        else
          tempDevice->setOnBoot("no");
      }
      else if ( node.nodeName() == "bootproto" )
      {
        QDomElement e = node.toElement();
        tempDevice->setBootProto(e.text());
      }
      else if ( node.nodeName() == "dev" )
      {
        QDomElement e = node.toElement();
        tempDevice->setDeviceName(e.text());
      }
      //we had to add the OR because the xml sintax when listing the interfaces
      //is different than when loading the network info. ie.: enabled->active
      //address->addr, etc...
      else if ( node.nodeName() == "enabled"  || node.nodeName() == "active")
      {
        QDomElement e = node.toElement();
        tempDevice->setActive(e.text());
      }
      if ( node.nodeName() == "address" || node.nodeName() == "addr")
      {
        QDomElement e = node.toElement();
        tempDevice->setIpAddress(e.text());
      }
      else if ( node.nodeName() == "netmask" || node.nodeName() == "mask")
      {
        QDomElement e = node.toElement();
        tempDevice->setNetmask(e.text());
      }
      else if ( node.nodeName() == "network" )
      {
        QDomElement e = node.toElement();
        tempDevice->setNetwork(e.text());
      }
      else if ( node.nodeName() == "broadcast" || node.nodeName() == "bdcast")
      {
        QDomElement e = node.toElement();
        tempDevice->setBroadcast(e.text());
      }      
      else if ( node.nodeName() == "name" )
      {
        QDomElement e = node.toElement();
        if (e.text() != "")
          tempDevice->setDescription(e.text());
        else
          tempDevice->setDescription(i18n("Ethernet Network Device"));
      }

    }
    node = node.nextSibling();
  }
  //Clear IP address settings if boot protocol es dhcp or bootp, in case that they are
  //setted in the config files.
  if ((tempDevice->getBootProto().lower() == "dhcp") || (tempDevice->getBootProto().lower() == "bootp"))
  {
    if (tempDevice->getIpAddress() == "" && tempDevice->isActive())
      tempDevice->setActive("0");

    tempDevice->setIpAddress("");
    tempDevice->setNetmask("");
    tempDevice->setNetwork("");
    tempDevice->setBroadcast("");
  }

  return tempDevice;
}
/** Reads the xml with the network info. */
void KNetworkConfigParser::readNetworkInfo(){
  dnsInfo = NULL;
  routingInfo = NULL;
  QPtrList<KNetworkInterface> tempDeviceList;
  routingInfo = new KRoutingInfo();
  dnsInfo = new KDNSInfo();

  deviceList.clear();
  //The gst backend puts a \n at the beginning of the xml output, so
  //we have to erase it first before we parse it.
  xmlOuput = xmlOuput.section('\n',1);

  if (xmlErr.contains("platform_unsup::"))
  {
    connect( this, SIGNAL(readyLoadingSupportedPlatforms()), this, SLOT(showSupportedPlatformsDialogSlot()) );
    loadSupportedPlatforms();
    emit errorDetectingPlatform();
  }
  else
  {
    QString err;
    int x,y;
    QDomDocument doc( "network");
    if ( !doc.setContent( xmlOuput.utf8(),false,&err,&x,&y  ) )    
    {
      KMessageBox::error(0,
									i18n("Couldn't parse the XML file."),
                      i18n("Error"));
        qDebug("error: %s %d,%d",err.latin1(),x,y);
    }
    dnsInfo = loadDNSInfo();
    QDomElement root = doc.documentElement();
    QDomNode node = root.firstChild();
    QString s;
    while( !node.isNull() )
    {
      if ( node.isElement() && node.nodeName() == "gateway" )
      {
        QDomElement gateway = node.toElement();
        routingInfo->setGateway(gateway.text());
      }
      else if ( node.isElement() && node.nodeName() == "gatewaydev" )
      {
        QDomElement gatewaydev = node.toElement();
        routingInfo->setGatewayDevice(gatewaydev.text());
      }
      else if ( node.isElement() && node.nodeName() == "interface" )
      {
        QDomElement interface = node.toElement();
        KNetworkInterface *tempDevice = new KNetworkInterface();
        tempDevice = getInterfaceInfo(interface);
        //Don't include the loopback device
        if (tempDevice->getDeviceName().lower() != "lo")
          deviceList.append(tempDevice);
      }
      else if ( node.isElement() && node.nodeName() == "hostname" )
      {
        QDomElement hostname = node.toElement();
        dnsInfo->setMachineName(hostname.text());
      }
      else if ( node.isElement() && node.nodeName() == "domain" )
      {
        QDomElement domainname = node.toElement();
        dnsInfo->setDomainName(domainname.text());
      }      
      node = node.nextSibling();
    }
    loadDhcpDevices();
    networkInfo->setDeviceList(deviceList);
    loadRoutingInfo();
    networkInfo->setRoutingInfo(routingInfo);
    networkInfo->setDNSInfo(dnsInfo);
    qDebug("--get XML:\n%s",xmlOuput.latin1());
    //Tell to anyone interested when the network info is loaded.
    emit readyLoadingNetworkInfo();
    }
}
/** Calls the network scripts to reload the network depending on the system it's running on, so the configuration changes takes effect. */
bool KNetworkConfigParser::reloadNetwork(){
  procReloadNetwork = new QProcess(this);
//  kpdReloadStatusDlg = new KReloadNetworkDlg(this);
    kpdReloadStatusDlg = new KInterfaceUpDownDlg(0,"Reloading the Network");
    kpdReloadStatusDlg->label->setText("Please wait while the network is restarted...");
//  isUp = false;
    procReloadNetwork->addArgument( "/etc/init.d/network" );
    procReloadNetwork->addArgument( "restart" );

  connect(procReloadNetwork,SIGNAL(readyReadStdout()),this,SLOT(readFromStdoutReloadScript()));
  connect(procReloadNetwork,SIGNAL(processExited()),this,SLOT(closeReloadStatusWindowSlot()));
  connect(procReloadNetwork,SIGNAL(processExited()),kpdReloadStatusDlg,SLOT(close()));
//  connect(kpdReloadStatusDlg->cancelButton, SIGNAL(clicked()), procReloadNetwork, SLOT(tryTerminate()));

  kpdReloadStatusDlg->show();
  if ( !procReloadNetwork->start() )
  {
    // error handling
//    kpdReloadStatusDlg->kpbOk->setEnabled(true);
    //kpdReloadStatusDlg->tlNetworkStatus->setText("There was a problem reloading the network");
    kpdReloadStatusDlg->label->setText("There was a problem reloading the network");
  }
  if (reloadScriptOutput.contains("failed"))
    return false;
  else
    return true;
}
void KNetworkConfigParser::readFromStdoutReloadScript(){
  QString s = procReloadNetwork->readStdout();
  reloadScriptOutput.append(s);
}
void KNetworkConfigParser::closeReloadStatusWindowSlot(){
  emit network_reloaded();
}
/** emits a signal when the network changes have benn saved. */
void KNetworkConfigParser::sendNetworkInfoSavedSignalSlot(){
//  loadDhcpDevices();  
  procSaveNetworkInfo->closeStdin();
//  emit readySavingNetworkInfo();
}
/** Concatenates into a QString the xml output of the network backend. */
void KNetworkConfigParser::concatXMLOutputSlot(){
  xmlOuput.append(procDetect->readStdout());
}

void KNetworkConfigParser::readXMLErrSlot(){
  xmlErr.append(procDetect->readStderr());
}
/** lists all platforms supported by GST. */
void KNetworkConfigParser::loadSupportedPlatforms(){
	procDetect = new QProcess(this);
  procDetect->addArgument( locate("data",BACKEND_PATH) );
  procDetect->addArgument( "-d" );  
  procDetect->addArgument( "platforms" );
  connect( procDetect, SIGNAL(processExited()), this, SLOT(readSupportedPlatformsSlot()) );
  xmlOuput = "";
  connect( procDetect, SIGNAL(readyReadStdout()),this, SLOT(concatXMLOutputSlot()));
//  connect( procDetect, SIGNAL(readyReadStderr()),this, SLOT(readXMLErrSlot()));

 	if ( !procDetect->start() )
  {
// error handling
    KMessageBox::error(0,
                        i18n("Couldn't list supported platforms. Something is wrong with your installation."),
                        i18n("Couldn't launch network backend script"));
    exit(5);
	}

}
/** Parses the xml ouput generated by GST that has all the supported platforms. */
void KNetworkConfigParser::readSupportedPlatformsSlot(){
  //The gst backend puts a \n at the beginning of the xml output, so
  //we have to erase it first before we parse it.
  xmlOuput = xmlOuput.section('\n',1);
  QDomDocument doc( "platforms" );
  if ( !doc.setContent( xmlOuput.utf8() ) )
  {
    KMessageBox::error(0,
									i18n("Couldn't parse the XML file of supported platforms."),
                      i18n("Error"));
  }
  QDomElement root = doc.documentElement();
  QDomNode node = root.firstChild();
  QString s;
  while( !node.isNull() )
  {
    if ( node.isElement() && node.nodeName() == "platform" )
    {
      QDomElement platform = node.toElement();
      s = getPlatformInfo(platform);
    }
    supportedPlatformsList << s;
    node = node.nextSibling();
  }
  emit readyLoadingSupportedPlatforms();
}
/** Returns the info of a platform in the form of 'key:value' . */
QString KNetworkConfigParser::getPlatformInfo(QDomElement platform){
  QDomNode node = platform.firstChild();
  QString s;
  while ( !node.isNull() )
  {
    if ( node.isElement() )
    {
      if ( node.nodeName() == "key" )
      {
        QDomElement e = node.toElement();
        s += e.text();
        s += ":";
      }
      else if ( node.nodeName() == "name" )
      {
        QDomElement e = node.toElement();
        s += e.text();
      }

    }
    node = node.nextSibling();
  }

  return s;
}
/** Shows the dialog with all the supported platforms by GST. */
void KNetworkConfigParser::showSupportedPlatformsDialogSlot(){
  KSelectDistroDlg* dialog = new KSelectDistroDlg(0, 0);

  for ( QStringList::Iterator it = supportedPlatformsList.begin(); it != supportedPlatformsList.end(); ++it )
  {
    QString key = (*it);
    key = key.section(":",0,0);
    QString name = (*it);
    name = name.section(":",1,1);
    if (key.contains("debian"))
    {
      QPixmap distroImg(locate("data","knetworkconf/pixmaps/debian.png"));
      dialog->klbDistroList->insertItem(distroImg, name);
    }
    else if (key.contains("mandrake"))
    {
      QPixmap distroImg(locate("data","knetworkconf/pixmaps/mandrake.png"));
      dialog->klbDistroList->insertItem(distroImg, name);
    }
    else if (key.contains("pld"))
    {
      QPixmap distroImg(locate("data","knetworkconf/pixmaps/pld.png"));
      dialog->klbDistroList->insertItem(distroImg, name);
    }
    else if (key.contains("redhat"))
    {
      QPixmap distroImg(locate("data","knetworkconf/pixmaps/redhat.png"));
      dialog->klbDistroList->insertItem(distroImg, name);
    }
    else if (key.contains("suse"))
    {
      QPixmap distroImg(locate("data","knetworkconf/pixmaps/suse.png"));
      dialog->klbDistroList->insertItem(distroImg, name);
    }
    else if (key.contains("turbolinux"))
    {
      QPixmap distroImg(locate("data","knetworkconf/pixmaps/turbolinux.png"));
      dialog->klbDistroList->insertItem(distroImg, name);
    }
    
  } 
  if (!dialog->exec())
    kapp->quit();
  else
  {
    int i = 0;
    QStringList::Iterator it = supportedPlatformsList.begin();
    while (i < dialog->klbDistroList->currentItem())
    {
      i++;
      ++it;
    }
    QString key = (*it);
    key = key.section(":",0,0); 
    //clean variables and run again the detection script but now don't
    //auto-detect.
    xmlErr = "";
    xmlOuput = "";
    networkInfo->setPlatformName(key);
    runDetectionScript(key);
  }  
}
/** No descriptions */
KNetworkInfo * KNetworkConfigParser::getNewNetworkInfo(){
  return networkInfo;
}
/** No descriptions */
void KNetworkConfigParser::assignNewDeviceList(){
  networkInfo->setDeviceList(deviceList);
  QPtrList<KNetworkInterface> l = deviceList;
  KNetworkInterface *dev;
  dev = l.first();

}

/** No descriptions */
void KNetworkConfigParser::readFromStdoutSaveNetworkInfo(){
  xmlOuput.append(procSaveNetworkInfo->readStdout());
}
/** Calls runDetectionScript(). */
void KNetworkConfigParser::runDetectionScriptSlot(){
  listIfaces(networkInfo->getPlatformName());
}
